//---------------------------------------------------------------------------------------------------------------------
//
// Project: MAX7456 OSD Device Driver
//
// Component: ARM version
//			  
//
// Started: Dec 07, Anthony Cake, ARM version Cdric Marzer
//
// NOTE: This code is formatted with 4 character tab spacing, not 3. This may be
// changed in most editors (for CCS it's under options->editor properties)
//
// Notes:
//		
//---------------------------------------------------------------------------------------------------------------------

#include <string.h>

#define NUM_COLS		30
#define NUM_ROWS_PAL	16
#define NUM_ROWS_NTSC	13
#define NUM_ROWS		NUM_ROWS_PAL

#define POWER_ON_RESET_TIME	100		// datasheet says 50ms, double it to be sure (in ms of course)
#define GENERAL_WAIT_TIME	1		// all of the clk-to-data, CS to clk, etc. timings are < 1us, so just use 1us everywhere to keep things simple

// definitions of the various registers in the Maxim chip, including definitions of some of the bits within them
//

#define READ_OFFSET     0x80		// to turn a write address, into a read address, add this

#define MAX7456_VM0     0x00
#define MAX7456_VM0_PAL     0x40    // 0 = NTSC, 1 = PAL
#define MAX7456_VM0_SYNC1   0x20    // Sync, 00 = Auto, 10 = External, 11 = Internal
#define MAX7456_VM0_SYNC2   0x10    // 
#define MAX7456_VM0_ENAB    0x08    // 1 = On, 0 = Off
#define MAX7456_VM0_VSYNC   0x04    // 1 = wait for sync
#define MAX7456_VM0_RST     0x02    // 1 = SW Reset
#define MAX7456_VM0_VBE     0x01    // 1 = Disable Video Buffer

#define MAX7456_VM1     0x01
#define MAX7456_VM1_BKMOD   0x80	// background mode, 0 = local control, 1 = all displayed background colors gray
#define MAX7456_VM1_BKBRI2  0x40	// background mode brightness (% of OSD white level)
#define MAX7456_VM1_BKBRI1  0x20
#define MAX7456_VM1_BKBRI0  0x10
#define MAX7456_VM1_BLTIM1  0x08	// Blink time (00 = 33/40ms, 01 = 67/80ms, 10 = 100/120ms, 11 = 133ms/160ms)
#define MAX7456_VM1_BLTIM0  0x04	
#define MAX7456_VM1_BLCYC1  0x02	// Blinking duty cycle (00 = BT:BT, 01 = BT:2BT, 10 = BT:3BT, 11 = 3BT:BT)
#define MAX7456_VM1_BLCYC0  0x01

#define MAX7456_HOS     0x02		// Horizontal Offset
#define MAX7456_VOS     0x03		// Vertical Offset

#define MAX7456_DMM     0x04		// Display Memory Mode
#define MAX7456_DMM_8BIT    0x40    // 1 = 8-bit mode, 0 = 16-bit mode
#define MAX7456_DMM_LBC     0x20	// Local Background Control
#define MAX7456_DMM_BLK     0x10	// Blink Bit (0 = off, 1, = on)
#define MAX7456_DMM_INV     0x08	// Invert (0 = normal, 1 = Invert)
#define MAX7456_DMM_CLR     0x04	// Clear display memory
#define MAX7456_DMM_VSCLR   0x02	// Vertical Sync Clear
#define MAX7456_DMM_AINC    0x01	// Auto_increment mode (0 = Disabled, 1 = Enabled)

#define MAX7456_DMAH    0x05		// Display Memory Address, High Byte
#define MAX7456_DMAH_BYTSEL 0x02	// Byte Select (0 = Character Address Byte, 1 = Character Attribute Byte)
#define MAX7456_DMAH_BIT8   0x01	// Bit 8 of the address defined in DMAL

#define MAX7456_DMAL    0x06		// Display Memory Address, Low Byte 
#define MAX7456_DMDI    0x07		// Display Memory Data In
#define MAX7456_CMM     0x08		// Character Memory Mode 
#define MAX7456_CMAH    0x09		// Character Memory Address, High Byte
#define MAX7456_CMAL    0x0a		// Character Memory Address, Low Byte
#define MAX7456_CMDI    0x0b		// Character Memory Data In (defines 4 pixels)
#define MAX7456_OSDM    0x0c		// OSD Insertion Mux (Rise/Fall time)
#define MAX7456_RB0     0x10		// Row 0 Brightness (Black & White Pixels)
#define MAX7456_RB1     0x11		// Row 1 Brightness
#define MAX7456_RB2     0x12	
#define MAX7456_RB3     0x13
#define MAX7456_RB4     0x14
#define MAX7456_RB5     0x15
#define MAX7456_RB6     0x16
#define MAX7456_RB7     0x17
#define MAX7456_RB8     0x18
#define MAX7456_RB9     0x19
#define MAX7456_RB10    0x1a
#define MAX7456_RB11    0x1b
#define MAX7456_RB12    0x1c
#define MAX7456_RB13    0x1d
#define MAX7456_RB14    0x1e
#define MAX7456_RB15    0x1f
#define MAX7456_OSDBL   0x6c		// OSD Black Level Control

#define MAX7456_STAT    0xa0		// Status Register
#define MAX7456_DMDO    0xb0		// Display Memory Data Out
#define MAX7456_CMDO    0xc0		// Character Memory Data Out
 
// character attributes
//
#define ATTRIB_NORMAL	0x00
#define ATTRIB_LBC		0x80
#define ATTRIB_BLINK	0x40
#define ATTRIB_INVERT	0x20

// 'special' character mapping
//
#define OSD_Small_M			0xa0
#define OSD_Small_V			0xa1
#define OSD_Small_W			0xa2
#define OSD_Small_A			0xa3
#define OSD_endstopArrow_L	0xbe	
#define OSD_endstopArrow_R	0xbf
#define OSD_alt_L			0xc0
#define OSD_alt_R			0xc1
#define OSD_bat_L			0xc2
#define OSD_bat_R			0xc3
#define OSD_cur_L			0xc4
#define OSD_cur_R			0xc5
#define OSD_dst_L			0xc6
#define OSD_dst_R			0xc7
#define OSD_mAh_L			0xc8
#define OSD_mAh_R			0xc9
#define OSD_kph_L			0xca
#define OSD_kph_R			0xcb
#define OSD_mph_L			0xcc
#define OSD_mph_R			0xcd
#define OSD_km_L			0xce
#define OSD_km_R			0xcf
#define OSD_mi_L			0xb0
#define OSD_mi_R			0xb1
#define OSD_spd_L			0xb2
#define OSD_spd_R			0xb3
#define OSD_ft				0xb4
#define OSD_sat				0xb5
#define OSD_acquiring0		0xb6
#define OSD_acquiring4		0xba
#define OSD_satellites0		0x87
#define OSD_satellites6		0x8d

#define OSD_rateOfClimb0	0x99
#define OSD_rateOfClimb1	0x9a
#define OSD_rateOfClimb2	0x9b
#define OSD_rateOfClimb3	0x9c		// neither climbing or falling
#define OSD_rateOfClimb4	0x9d
#define OSD_rateOfClimb5	0x9e
#define OSD_rateOfClimb6	0x9f


#define OSD_arrow_0			0x87
#define OSD_arrow_45		0x88
#define OSD_arrow_90		0x89
#define OSD_arrow_135		0x90
#define OSD_arrow_180		0x91
#define OSD_arrow_225		0x92
#define OSD_arrow_270		0x93

#define OSD_endstop_L	0xd0
#define OSD_plainBar	0xd1
#define OSD_endstop_R	0xd2
#define OSD_tick_0		0xd3
#define OSD_tick_1		0xd4
#define OSD_tick_2		0xd5
#define OSD_tick_3		0xd6
#define OSD_tick_4		0xd7
#define OSD_H_L			0xd8
#define OSD_H_0			0xd9		// must be prefixed with H_L
#define OSD_H_1			0xda
#define OSD_H_2			0xdb
#define OSD_H_3			0xdc
#define OSD_H_4			0xdd		// must be followed by H_R
#define OSD_H_R			0xde

#define OSD_Compass0L	0xe0
#define OSD_Compass0R	0xe1
#define OSD_Compass14L	0xfc
#define OSD_Compass14R	0xfd

#define CHAR_BITMAP_SIZE	54

#define int16 unsigned short int
#define int8 unsigned char
 
#define SPI_OK         0           	// transfer ended No Errors 
#define SPI_BUSY       1           	// transfer busy 
#define SPI_ERROR      2           	// SPI error 

// enum used to describe the state of the OSD's video standard detection
typedef enum { MAX7456_StdDetectLOS, MAX7456_StdDetectPAL, MAX7456_StdDetectNTSC } MAX7456_StdDetect;

int tvStandard = MAX7456_VM0_PAL;		// values of 0, or MAX7456_VM0_PAL
//int tvStandard = 0;
 
static unsigned char   state;      	// State of SPI driver 
static unsigned char   spiBuf[4];  	// SPI data buffer 
static unsigned char   *msg;       	// pointer to SPI data buffer 
static unsigned char   count;      	// nr of bytes send/received 
 
//-----------------------------------------------------------------------------------------------------
// NOTE: This ISR has not yet been proven to work. Polled mode replaces it for now

/*
void SPI_Isr (void) __attribute__ ((interrupt("IRQ")));
void SPI_Isr(void)
{ 
    if ((S0SPSR & 0xF8) == 0x80) 
    { 
        *msg++ = S0SPDR;           // read byte from slave 
        if (--count > 0) 
            S0SPDR = *msg;         // sent next byte 
        else 
            state = SPI_OK;        // transfer completed 
    } 
    else                           // SPI error 
    { 
        *msg = S0SPDR;             // dummy read to clear flags 
        state = SPI_ERROR; 
    } 
    S0SPINT = 0x01;                // reset interrupt flag 
    VICVectAddr = 0;               // reset VIC 
} 
 */
 
//-----------------------------------------------------------------------------------------------------
// Write a single byte to the OSD, via SPI, at the specified address
static void MAX7456_Write(unsigned char add, unsigned char val) 
{ 
    spiBuf[0] = add;               // OSD address 
    spiBuf[1] = val; 
    msg = spiBuf; 
    count = 2;                     // nr of bytes 
    state = SPI_BUSY;              // Status of driver 
 
    IOCLR0 = 0x00000080;           // OSD CS = 0
    S0SPDR = *msg++;                 // sent first byte 
    //while (state == SPI_BUSY) ;    // wait for end of transfer
    
     while((S0SPSR & 0x80) == 0)
     	;
     	
    S0SPDR = *msg;                 // sent second byte 
	while((S0SPSR & 0x80) == 0)
 		;
      
    IOSET0 = 0x00000080;           // OSD CS = 1 
} 
 


//-----------------------------------------------------------------------------------------------------
// Initialize SPI on the LPC2148
// Note that the SPI setup is really device-dependent, so it doesn't really make sense to make this more 
// general purpose. The CPHA, CPOL, and rate can all change between devices
static void SPI_Init(void) 
{ 
	// Configure the interrupt controller
	    //VICVectAddr0 = (unsigned int) &SPI_Isr; 
	    //VICVectCntl0 = 0x2A;           // Channel0 on Source#10 ... enabled 
	    //VICIntEnable |= 0x400;         // 10th bit is the SPI 
 
		//install_irq( SPI0_INT, (void *)SPI_Isr);
		
 	// Setup the SPI port
	    IODIR0  |= 0x00000080;         // 1 = output, P7 = SSEL0. all others input (why? are they managed by the SPI interface?) 
	    IOSET0   = 0x00000080;         // set SSEL0 = 1 (OSD CS = 1) 
	    PINSEL0 |= 0x00001500;         // configure SPI0 pins (except SSEL0) 
 
 	// Setup the SPI clock rate
    	S0SPCCR = 12;                  // SCK = 1 MHz, counter > 8 and even 
    
    // setup for CPHA = 0 (first edge of clock), and CPOL = 0 (sclk is active high), master mode, interrupts enabled
    // 8 bit transfers
    	S0SPCR  = 0xA0;                   
} 

//------------------------------------------------------------------------------------------
// Erase the display contents
void MAX7456_ClearScreen()
{
    MAX7456_Write(MAX7456_DMM, MAX7456_DMM_CLR | MAX7456_DMM_8BIT);
}

//------------------------------------------------------------------------------------------
// Write a single character (OSD's character set)
// NOTE: X, Y are zero-based

void MAX7456_WriteChar(int x, int y, unsigned char chr, int attribs )
{
	int16 addr; 
	unsigned char msb;
	addr = (int16) x + ((int16) y * NUM_COLS);
	msb = 0;
	
	if(addr > 255)
		msb = 1;
    MAX7456_Write(MAX7456_DMAH, msb);			// MAX7456_DMAH_BYTSEL = 0, MSBs of address = 0
   	MAX7456_Write(MAX7456_DMAL, addr & 0xff);	 
   	MAX7456_Write(MAX7456_DMDI, chr);			// character code

	//if(attribs > 0)
	{
		msb |= 0x02;							// DMDI will write char. attribute
    	MAX7456_Write(MAX7456_DMAH, msb);		// MAX7456_DMAH_BYTSEL = 0, MSBs of address = 0
   		MAX7456_Write(MAX7456_DMDI, attribs);	// character attributes
	}
}


//------------------------------------------------------------------------------------------
// Translate an ASCII character into an OSD code
//
unsigned char MAX7456_ASCII2OSDCode(unsigned char ascii)
{
	int OSDCodeOffset = 0x3f;
	unsigned char ASCIICodes[] = "().?;:,\'/\"-<>@";
	unsigned char chr;
	
	if(ascii == ' ')
		chr = 0x00;
	else if(ascii == '0')
		chr = 0x0a;
	else if(ascii >= '1' && ascii <= '9')
		chr = ascii - '1' + 0x01;
	else if(ascii >= 'A' && ascii <= 'Z')
		chr = ascii - 'A' + 0x0b;
	else if(ascii >= 'a' && ascii <= 'z')
		chr = ascii - 'a' + 0x25;
	else
	{
		int i;
		// lookup the ASCII chacter in the list of codes, and compute its OSD equivalent
		for(i = 0; i < sizeof(ASCIICodes); ++i)
			if(ascii == ASCIICodes[i])
			{
				chr = OSDCodeOffset + i;
				break;
			}
			
		// not found?
			if(i == sizeof(ASCIICodes))
			{
				chr = ascii;
			}
	}
	
	return chr;
}



//------------------------------------------------------------------------------------------
// Write a single character, using the ASCII character set
// NOTE: X, Y are zero-based
void MAX7456_WriteASCII(int x, int y, unsigned char ascii, int attribs )
{
	unsigned char chr;
	chr = MAX7456_ASCII2OSDCode(ascii);
	
	MAX7456_WriteChar(x, y, chr, attribs);
}

//------------------------------------------------------------------------------------------
void MAX7456_WriteString(int x, int y, char *string, int attribs )
{
	char *sP;
	sP = string;
	
	do
	{
		MAX7456_WriteASCII(x++, y, *sP, attribs);
	} while (*(++sP) != 0);
}

//------------------------------------------------------------------------------------------
// Readout the OSD character map, and dump to the USB (CDC) port, capture using hyperterminal
// (or similar)
void MAX7456_WriteCharMapForChr(int chr, unsigned char buf[54])
{
	int i;

 	MAX7456_Write(MAX7456_VM0, tvStandard);	// disable the OSD
	MAX7456_Write(MAX7456_CMAH, chr); 	    // character code

	for(i = 0; i < CHAR_BITMAP_SIZE; ++i)
	{
		MAX7456_Write(MAX7456_CMAL, i); 	
		MAX7456_Write(MAX7456_CMDI, buf[i]); 
	}
	MAX7456_Write(MAX7456_CMM, 0xa0); 		// read from NVM  
	delay_ms(20);							// wait for the write to occur (TODO: Read STAT here to be sure)
 	MAX7456_Write(MAX7456_VM0, MAX7456_VM0_ENAB | tvStandard);
	delay_ms(20);							
}

//------------------------------------------------------------------------------------------
void MAX7456_ClearLine(int y)
{
	int x;
	for(x = 0; x < NUM_COLS; ++x)
	{
		MAX7456_WriteASCII(x, y, ' ', 0);
	}
}

//------------------------------------------------------------------------------------------
void MAX7456_Init()
{
	SPI_Init();
	
	// enable the OSD, define the TV standard required
	    	MAX7456_Write(MAX7456_VM0, MAX7456_VM0_ENAB | tvStandard);

	    // enable 8-bit mode
	    	MAX7456_Write(MAX7456_DMM, MAX7456_DMM_8BIT);   

		// set the blink control
	    	MAX7456_Write(MAX7456_VM1, MAX7456_VM1_BKBRI2 | MAX7456_VM1_BLTIM0 | MAX7456_VM1_BLCYC1 | MAX7456_VM1_BLCYC0);

		// clear the display
			MAX7456_ClearScreen();
      

}

